# define variable 'ops': optimisation parameter set
# define variable 'setups': simulation setup set
# define variable 'optimiser': properties for the optimiser

import numpy as np
import pandas as pd
import math

# import just for IDE convenience
import propti as pr

# Inverse modelling with only higher heat fluxes (50 kW and 75 kW). The 25 kW
# have been removed, because they seemed to have a negative impact on best
# parameter set, and it could not be pushed to the desired quality.
# fix the chid

CHID = 'ML_CONE'

ConeHeatFluxes = [75]
template_file = ['Simple_Test_Alu.fds', 'Simple_Test_ISO.fds']

experimental_data_file_list = [['75_MLC_Alu.csv']]

experimental_data_file_list_iso = [['75_MLC_ISO.csv']]

exp_x_values = [['Time_Alu']]

exp_x_values_iso = [['Time_ISO']]

exp_y_values = [['Mass_Norm_Alu']]

exp_y_values_iso = [['Mass_Norm_ISO']]


# Round value up to next full value.
def roundup(x, full_value=10):
    return int(math.ceil(x / float(full_value))) * full_value


# Find last x values and their index in the CSV-files containing the
# experimental data. Also, determine the end time of the simulation.
end_of_time = []  # Simulation end time.
map_def_list = []  # Values for the mapping of sim data to exp data.
end_of_time_alu = []
map_def_list_alu = []
for i in range(len(experimental_data_file_list)):
    mdl1 = []
    eot = []

    iterator = len(experimental_data_file_list[i])
    for j in range(iterator):
        mdl2 = []
        print(experimental_data_file_list[i][j])
        print(exp_x_values[i][j])
        etc = pd.read_csv(experimental_data_file_list[i][j],
                          header=0)[exp_x_values[i][j]]

        etc_i = etc.last_valid_index()
        etc_v = etc[etc_i]
        mdl2.append(etc_i)
        mdl2.append(etc_v)

        mdl1.append(mdl2)

        eot.append(etc_v)

    # Calculating simulation end based on the largest time value from
    # experiment and round up to next full 100.
    max_val = max(eot)
    rv = roundup(max_val, 100)
    eot2 = []
    for k in range(iterator):
        eot2.append(rv)

    # Sim end.
    end_of_time_alu.append(eot2)

    # Data mapping.
    map_def_list_alu.append(mdl1)

# Collect end of time and map def lists.
end_of_time.append(end_of_time_alu)
map_def_list.append(map_def_list_alu)


end_of_time_iso = list()
map_def_list_iso = list()
for i in range(len(experimental_data_file_list_iso)):
    mdl1_iso = []
    eot_iso = []

    iterator = len(experimental_data_file_list_iso[i])
    for j in range(iterator):
        mdl2_iso = []
        print(experimental_data_file_list_iso[i][j])
        print(exp_x_values_iso[i][j])
        etc_iso = pd.read_csv(experimental_data_file_list_iso[i][j],
                          header=0)[exp_x_values_iso[i][j]]

        etc_i_iso = etc_iso.last_valid_index()
        etc_v_iso = etc_iso[etc_i_iso]
        mdl2_iso.append(etc_i_iso)
        mdl2_iso.append(etc_v_iso)

        mdl1_iso.append(mdl2_iso)

        eot_iso.append(etc_v_iso)

    # Calculating simulation end based on the largest time value from
    # experiment and round up to next full 100.
    max_val_iso = max(eot_iso)
    rv_iso = roundup(max_val_iso, 100)
    eot2_iso = []
    for k in range(iterator):
        eot2_iso.append(rv_iso)

    # Sim end.
    end_of_time_iso.append(eot2_iso)

    # Data mapping.
    map_def_list_iso.append(mdl1_iso)

# Collect end of time and map def lists.
end_of_time.append(end_of_time_iso)
map_def_list.append(map_def_list_iso)

# define the optimisation parameter
op1 = pr.Parameter(name='EMISSIVITY', place_holder='EMISSIVITY',
                   min_value=0.4, max_value=0.99)
op2 = pr.Parameter(name='DENSITY', place_holder='DENSITY',
                   min_value=10, max_value=2000)
op3 = pr.Parameter(name='SPECIFIC_HEAT', place_holder='SPECIFIC_HEAT',
                   min_value=0.01, max_value=2)
op4 = pr.Parameter(name='A', place_holder='A',
                   min_value=1e-1, max_value=1e+3)
op5 = pr.Parameter(name='E', place_holder='E',
                   min_value=1e+2, max_value=1e+6)
op6 = pr.Parameter(name='HEAT_OF_COMBUSTION', place_holder='HEAT_OF_COMBUSTION',
                   min_value=1e+4, max_value=6e+4)
op7 = pr.Parameter(name='HEAT_OF_REACTION', place_holder='HEAT_OF_REACTION',
                   min_value=100, max_value=1e+4)
op8 = pr.Parameter(name='N', place_holder='N',
                   min_value=0.01, max_value=1)
op9 = pr.Parameter(name='CONDUCTIVITY', place_holder='CONDUCTIVITY',
                   min_value=0.01, max_value=2)


set_of_parameters = [op1, op2, op3, op4, op5, op6, op7, op8, op9]

# Definition of parameters which is used by propti_prepare.py, it has no
# further meaning here.
ops = pr.ParameterSet(params=set_of_parameters)
# define general model parameter, including optimisation parameter
# Function to provide basic parameters for one simulation setup.

def create_mod_par_setup(iter, eot_list, alu_oder_iso):
    # Provide optimisation parameters to the model parameter setups.
    ps = pr.ParameterSet(params=set_of_parameters)

    # Add different heat fluxes (20,30,40, 50, 75).
    ps.append(pr.Parameter(name='ConeFlux' + str(ConeHeatFluxes[iter]),
                           place_holder='cone_flux',
                           value=ConeHeatFluxes[iter]))

    # Add individual character ID to distinguish the simulation data.
    ps.append(pr.Parameter(name='chid',
                           place_holder='chid',
                           value=CHID + '_{}_'.format(alu_oder_iso) + str(ConeHeatFluxes[iter]) + 'kW'))

    # Add individual simulation end time.
    ps.append(pr.Parameter(name='TEND',
                           place_holder='tend',
                           value=str(eot_list[iter][0])))

    ps.append(pr.Parameter(name='mesh_i', place_holder='i', value=6))
    ps.append(pr.Parameter(name='mesh_i', place_holder='j', value=6))
    ps.append(pr.Parameter(name='mesh_i', place_holder='k', value=9))
    return ps


# Calls the above function to create multiple parameter sets for the different
# simulation setups. The parameter sets (objects) are then stored in a list.
model_parameter_setups_alu = [create_mod_par_setup(i, end_of_time[0], "Alu") for i in
                              range(len(ConeHeatFluxes))]

model_parameter_setups_alu.append(pr.ParameterSet(params=set_of_parameters))


model_parameter_setups_iso = [create_mod_par_setup(i, end_of_time[1], "ISO") for i in
                              range(len(ConeHeatFluxes))]

model_parameter_setups_iso.append(pr.ParameterSet(params=set_of_parameters))

# Create a list of relations between experimental and model (simulation) data,
# for each experimental data series.


r = []
for i in range(len(exp_x_values)):
    ri = []
    for j in range(len(exp_x_values[i])):
        relation = pr.Relation()
        relation.model.file_name = "{}_devc.csv".format(
            CHID + '_Alu_' + str(ConeHeatFluxes[i]) + 'kW')
        relation.model.label_x = 'Time'
        relation.model.label_y = 'ML'
        relation.model.factor = 100
        relation.model.header_line = 1
        relation.experiment.file_name = \
            experimental_data_file_list[i][j]
        relation.experiment.label_x = exp_x_values[i][j]
        relation.experiment.label_y = exp_y_values[i][j]
        relation.experiment.header_line = 0
        # relation.experiment.factor = 1e-3
        relation.x_def = np.linspace(0., map_def_list[0][i][j][1], 540)  # Scale
        # with amount of data points in the shortest list (75kW). For testing
        # purpose fixed now to 540 data points, later likely to be scaled
        # differently.
        ri.append(relation)
    r.append(ri)


r2 = []
for i in range(len(exp_x_values_iso)):
    ri = []
    for j in range(len(exp_x_values_iso[i])):
        relation = pr.Relation()
        relation.model.file_name = "{}_devc.csv".format(
            CHID + '_ISO_' + str(ConeHeatFluxes[i]) + 'kW')
        relation.model.label_x = 'Time'
        relation.model.label_y = 'ML'
        relation.model.factor = 100
        relation.model.header_line = 1
        relation.experiment.file_name = \
            experimental_data_file_list_iso[i][j]
        relation.experiment.label_x = exp_x_values_iso[i][j]
        relation.experiment.label_y = exp_y_values_iso[i][j]
        relation.experiment.header_line = 0
        # relation.experiment.factor = 1e-3
        relation.x_def = np.linspace(0., map_def_list[1][i][j][1], 540)  # Scale
        # with amount of data points in the shortest list (75kW). For testing
        # purpose fixed now to 540 data points, later likely to be scaled
        # differently.
        ri.append(relation)
    r2.append(ri)


ssetups = []

for i in range(len(exp_x_values)):

    sj = pr.SimulationSetup(name=CHID + '_Alu_' + str(ConeHeatFluxes[
                                                     i]) + 'kW',
                            execution_dir_prefix='/dev/shm/',
                            #execution_dir_prefix='.',
                            # execution_dir_prefix='/work/jjsc27/jjsc2702/MLC/FDS_6.7/FDS_work',
                            work_dir=CHID + '_Alu_' +
                                     str(ConeHeatFluxes[i]) +
                                     'kW_run',
                            model_template=template_file[0],
                            model_parameter=model_parameter_setups_alu[i],
                            #model_executable='OMP_NUM_THREADS=1 mpiexec -n 1 fds',
                           # model_executable= "env -i bash -l -c 'module purge && module load FDS/6.7.0-IntelCompiler_2019.0_ParaStationMPI_5.2.1 && OMP_NUM_THREADS=1 fds model_input.file'",
                            model_executable=os.path.join(os.getcwd(), 'fds.jureca.sh'),
                            relations=r[i])

    ssetups.append(sj)

for i in range(len(exp_x_values_iso)):
    sj = pr.SimulationSetup(name=CHID + '_ISO_' + str(ConeHeatFluxes[
                                                     i]) + 'kW',
                             execution_dir_prefix='/dev/shm/',
                            #execution_dir_prefix='.',
                            #execution_dir_prefix='/work/jjsc27/jjsc2702/MLC/FDS_6.7/FDS_work',
                            work_dir=CHID + '_ISO_' +
                                     str(ConeHeatFluxes[i]) +
                                     'kW_run',
                            model_template=template_file[1],
                            model_parameter=model_parameter_setups_iso[i],
                            #model_executable='OMP_NUM_THREADS=1 mpiexec -n 1 fds',
                           # model_executable= "env -i bash -l -c 'module purge && module load FDS/6.7.0-IntelCompiler_2019.0_ParaStationMPI_5.2.1 && OMP_NUM_THREADS=1 fds model_input.file'",
                            model_executable=os.path.join(os.getcwd(), 'fds.jureca.sh'),
                            relations=r2[i])

    ssetups.append(sj)



# Initialise empty simulation setup sets.
setups = pr.SimulationSetupSet()

# append above object to simulation setup set
for i in range(len(exp_x_values)*2):
    setups.append(ssetups[i])


print('** setups generated')


# use default values for optimiser
optimiser = pr.OptimiserProperties(algorithm='fscabc',
                                   repetitions=180000,
                                   backup_every=20,
                                   eb=24,
                                   num_subprocesses=len(ConeHeatFluxes)*2,
                                   mpi=True)


print('** Input file processed.')
